# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import ethernet_interface
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestEthernetInterface(BaseFlaskTest):
    """Tests for EthernetInterface blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestEthernetInterface, self).setUpClass()

        self.app.register_blueprint(
            ethernet_interface.ethernet_interface)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerProfile.json'
        ) as f:
            self.server_profile = json.load(f)

    def test_get_ethernet_interface_when_it_has_only_one_vlan(self):
        """Tests request a valid EthernetInterface when it has one vlan"""

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'EthernetInterfaceWithOnlyOneVlan.json'
        ) as f:
            ethernet_interface_mockup = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'NetworkForEthernetInterface.json'
        ) as f:
            network = json.load(f)

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.index_resources.get.return_value = network

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/"
            "EthernetInterfaces/1"
        )

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(ethernet_interface_mockup, result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])

        # if verifies the URI of connection from ServerProfile.json mockup
        self.oneview_client.index_resources.get.assert_called_with(
            "/rest/ethernet-networks/19638712-679d-4232-9743-c7cb6c7bf718")

    def test_get_ethernet_interface_when_it_has_only_a_list_of_vlan(self):
        """Tests request a valid EthernetInterface when it has a vlans list"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'NetworkSetForEthernetInterface.json'
        ) as f:
            network_set = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'EthernetInterfaceWithListOfVlans.json'
        ) as f:
            ethernet_interface_mockup = json.load(f)

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.index_resources.get.return_value = network_set

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/"
            "EthernetInterfaces/2"
        )

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(ethernet_interface_mockup, result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])

        # if verifies the URI of connection from ServerProfile.json mockup
        self.oneview_client.index_resources.get.assert_called_with(
            "/rest/network-sets/76f584da-1f9d-40b8-9b9d-5ccb09810142")

    def test_get_ethernet_interface_when_id_is_not_found(self):
        """Tests request a not found EthernetInterface"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/"
            "EthernetInterfaces/3"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.index_resources.get.assert_not_called()

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/"
            "EthernetInterfaces/abc"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)

    def test_get_ethernet_interface_when_profile_not_found(self):
        """Tests request EthernetInterface when server profile not found"""

        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND'
        })

        self.oneview_client.server_profiles.get.side_effect = e

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/"
            "EthernetInterfaces/1"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.index_resources.get.assert_not_called()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.vlan_network_interface_collection \
    import VLanNetworkInterfaceCollection
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestVLanNetworkInterfaceCollection(BaseTest):
    """Tests for VLanNetworkInterfaceCollection class"""

    def setUp(self):
        """Tests preparation"""

        # Loading NetworkSet mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/NetworkSet.json'
        ) as f:
            self.network_set_mockup = json.load(f)

        # Loading VLanNetworkInterfaceCollection mockup value
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/VLanNetworkInterfaceCollectionSPT.json'
        ) as f:
            self.vlan_network_interface_collection = json.load(f)

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated
        vlan_network_interface_collection = VLanNetworkInterfaceCollection(
            self.network_set_mockup,
            self.vlan_network_interface_collection["@odata.id"])

        self.assertIsInstance(
            vlan_network_interface_collection, VLanNetworkInterfaceCollection)

    def test_serialize(self):
        # Tests the serialize function result against known result
        vlan_network_interface_collection = VLanNetworkInterfaceCollection(
            self.network_set_mockup,
            self.vlan_network_interface_collection["@odata.id"])

        result = json.loads(vlan_network_interface_collection.serialize())

        self.assertEqualMockup(self.vlan_network_interface_collection, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from flask import abort
from flask import Blueprint
from flask import g
from flask_api import status

from hpOneView.exceptions import HPOneViewException
from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.resource_block import ResourceBlock
from oneview_redfish_toolkit.api.resource_block_computer_system \
    import ResourceBlockComputerSystem
from oneview_redfish_toolkit.api.resource_block_ethernet_interface \
    import ResourceBlockEthernetInterface
from oneview_redfish_toolkit.api.server_hardware_resource_block \
    import ServerHardwareResourceBlock
from oneview_redfish_toolkit.api.server_profile_template_resource_block \
    import ServerProfileTemplateResourceBlock
from oneview_redfish_toolkit.api.storage_resource_block \
    import StorageResourceBlock
from oneview_redfish_toolkit.blueprints.util.response_builder \
    import ResponseBuilder
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit.services.manager_service import \
    get_manager_uuid
from oneview_redfish_toolkit.services.zone_service import ZoneService
from oneview_redfish_toolkit.single_oneview_context import single_oneview

resource_block = Blueprint("resource_block", __name__)


@resource_block.route(ResourceBlock.BASE_URI + "/<uuid>", methods=["GET"])
@single_oneview
def get_resource_block(uuid):
    """Get the Redfish ResourceBlock for a given UUID.

        Return ResourceBlock redfish JSON for a given UUID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock.
    """
    try:
        zone_service = ZoneService(g.oneview_client)
        resource = _get_oneview_resource(uuid)
        category = resource["category"]

        if category == "server-hardware":
            result_resource_block = _build_computer_system_resource_block(
                uuid, resource)

        elif category == "server-profile-templates":
            zone_ids = zone_service.get_zone_ids_by_templates([resource])
            if not zone_ids:
                raise OneViewRedfishError("Zone not found "
                                          "to ResourceBlock {}".format(uuid))

            result_resource_block = \
                ServerProfileTemplateResourceBlock(uuid, resource, zone_ids)

        elif category == "drives":
            drive_uuid = resource["uri"].split("/")[-1]
            drive_index_trees_uri = \
                "/rest/index/trees/rest/drives/{}?parentDepth=3"
            drive_index_trees = g.oneview_client.connection.get(
                drive_index_trees_uri.format(drive_uuid))

            server_profile_templs = \
                g.oneview_client.server_profile_templates.get_all()

            zone_ids = zone_service.get_zone_ids_by_templates(
                server_profile_templs)

            result_resource_block = StorageResourceBlock(
                resource, drive_index_trees, zone_ids)

        else:
            raise OneViewRedfishError('Resource block not found')

        return ResponseBuilder.success(
            result_resource_block,
            {"ETag": "W/" + resource["eTag"]})

    except OneViewRedfishError as e:
        # In case of error log exception and abort
        logging.exception('Unexpected error: {}'.format(e))
        abort(status.HTTP_404_NOT_FOUND, e.msg)


@resource_block.route(
    ResourceBlock.BASE_URI + "/<uuid>/Systems/1", methods=["GET"])
@single_oneview
def get_resource_block_computer_system(uuid):
    """Get Computer System of a Resource Block

        Return ResourceBlock Computer System redfish JSON for a given
        UUID. Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock Computer System.
    """

    server_hardware = g.oneview_client.server_hardware.get(uuid)
    manager_uuid = get_manager_uuid(uuid)

    computer_system = ResourceBlockComputerSystem(
        server_hardware, manager_uuid)

    return ResponseBuilder.success(
        computer_system,
        {"ETag": "W/" + server_hardware["eTag"]})


@resource_block.route(
    ResourceBlock.BASE_URI + "/<uuid>/EthernetInterfaces/<id>",
    methods=["GET"])
@single_oneview
def get_resource_block_ethernet_interface(uuid, id):
    """Get the Redfish ResourceBlock of type Network for the given UUID and ID.

        Return ResourceBlock redfish JSON for a given UUID and ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock.
    """

    try:
        server_profile_template = \
            g.oneview_client.server_profile_templates.get(uuid)

        conn_settings = server_profile_template["connectionSettings"]
        connection = None

        for conn in conn_settings["connections"]:
            if str(conn["id"]) == id:
                connection = conn
                break

        if not connection:
            raise OneViewRedfishError("Ethernet interface not found")

        network = \
            g.oneview_client.index_resources.get(connection["networkUri"])

        ethernet_interface = ResourceBlockEthernetInterface(
            server_profile_template, connection, network)

        return ResponseBuilder.success(
            ethernet_interface,
            {"ETag": "W/" + server_profile_template["eTag"]})

    except OneViewRedfishError as e:
        # In case of error log exception and abort
        logging.exception('Unexpected error: {}'.format(e))
        abort(status.HTTP_404_NOT_FOUND, e.msg)


def _get_oneview_resource(uuid):
    drives_param = "/rest/drives/" + uuid

    cached_categ = category_resource.get_category_by_resource_id(uuid) or \
        category_resource.get_category_by_resource_id(drives_param)

    if cached_categ:
        resource_uuid = uuid

        if cached_categ.resource == 'index_resources':
            resource_uuid = drives_param

        resource = getattr(g.oneview_client, cached_categ.resource)
        function = getattr(resource, cached_categ.function)

        return function(resource_uuid)

    categories = [
        {"func": g.oneview_client.server_hardware.get, "param": uuid},
        {"func": g.oneview_client.server_profile_templates.get, "param": uuid},
        {"func": g.oneview_client.index_resources.get, "param": drives_param}
    ]

    for category in categories:
        try:
            resource = category["func"](category["param"])

            return resource
        except HPOneViewException as e:
            if e.oneview_response["errorCode"] == 'RESOURCE_NOT_FOUND':
                pass
            else:
                raise  # Raise any unexpected errors

    raise OneViewRedfishError("Could not find resource block with id " + uuid)


def _build_computer_system_resource_block(uuid, server_hardware):
    eg_uri = server_hardware["serverGroupUri"]
    sht_uri = server_hardware["serverHardwareTypeUri"]

    filters = list()
    filters.append("enclosureGroupUri='{}'".format(eg_uri))
    filters.append("serverHardwareTypeUri='{}'".format(sht_uri))

    server_profile_templs = g.oneview_client \
        .server_profile_templates.get_all(filter=filters)

    zone_service = ZoneService(g.oneview_client)
    zone_ids = zone_service.get_zone_ids_by_templates(server_profile_templs)

    # Build ResourceBlock object and validates it
    return ServerHardwareResourceBlock(uuid, server_hardware, zone_ids)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for store_schema and load_registry function from util.py
"""

import json
import os
import socket

from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.event import Event
from oneview_redfish_toolkit.api.subscription import Subscription
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import util
import unittest
from unittest import mock


class TestUtil(unittest.TestCase):
    """Test class for util

        Tests:
            load_conf()
                - invalid ini file
                - valid ini file
                - have all expected sessions
                - have all expected options
                - checks if globals vars are not none

    """

    # load_conf() tests
    def setUp(self):
        self.schema_dir = './oneview_redfish_toolkit/schemas'
        self.registry_dir = './oneview_redfish_toolkit/registry'
        self.config_file = './oneview_redfish_toolkit/conf/redfish.conf'

    def test_get_ip(self):
        # Tests get_ip function; This test may not work if it returns an IPV6.
        ip = util.get_ip()
        try:
            socket.inet_aton(ip)
        except Exception:
            self.fail("Failed to get a valid IP Address")

    @mock.patch.object(connection, 'check_oneview_availability')
    def test_create_certs(
        self, check_ov_availability):
        # Test generate_certificate function

        config.load_config(self.config_file)

        util.generate_certificate("oneview_redfish_toolkit/",
                                  "test", 2048)

        self.assertTrue(os.path.exists(os.path.join("oneview_redfish_toolkit",
                        "test" + ".crt")))
        self.assertTrue(os.path.exists(os.path.join("oneview_redfish_toolkit",
                        "test" + ".key")))

    @mock.patch.object(connection, 'check_oneview_availability')
    def test_load_event_service_invalid_info(
        self, check_ov_availability):
        self.assertRaises(
            OneViewRedfishError, config.load_config(self.config_file))

    @mock.patch.object(connection, 'check_oneview_availability')
    @mock.patch.object(util, 'subscriptions_by_type')
    @mock.patch(
        'oneview_redfish_toolkit.event_dispatcher.EventDispatcher.start')
    def test_submit_event_with_subscriber(
        self, start_mock, subscription_mock, check_ov_availability):
        """Tests SubmitTestEvent action with two subscribers"""

        config.load_config(self.config_file)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Alert.json'
        ) as f:
            event_mockup = Event(json.loads(f.read()))

        subscription_mock['Alert'].values.return_value = [
            Subscription('1', 'destination1', [], 'context1'),
            Subscription('2', 'destination2', [], 'context2')
        ]

        util.dispatch_event(event_mockup)

        self.assertTrue(start_mock.call_count == 2)

    @mock.patch.object(connection, 'check_oneview_availability')
    @mock.patch.object(util, 'subscriptions_by_type')
    @mock.patch(
        'oneview_redfish_toolkit.event_dispatcher.EventDispatcher.start')
    def test_submit_event_without_subscriber(
        self, start_mock, subscription_mock, check_ov_availability):
        """Tests SubmitTestEvent action with no subscribers"""

        config.load_config(self.config_file)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Alert.json'
        ) as f:
            event_mockup = Event(json.loads(f.read()))

        subscription_mock['Alert'].values.return_value = []

        util.dispatch_event(event_mockup)

        self.assertFalse(start_mock.called)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api import status_mapping


class EthernetInterface(RedfishJsonValidator):
    """Creates a Ethernet Interface Redfish dict

        Populates self.redfish with some hardcoded EthernetInterface
        values and Oneview's Server Profile, Connection and Network
    """

    SCHEMA_NAME = 'EthernetInterface'

    def __init__(self, server_profile, connection, network_attr):
        """EthernetInterfaceCollection constructor

            Populates self.redfish and calls _validate()

            Args:
                server_profile: a server profile dict from OneView
                connection: a connection dict from server profile
                network_attr: a network (or networkSet) dict from
                OneView index resources
        """
        super().__init__(self.SCHEMA_NAME)

        conn_id = str(connection["id"])
        odata_uri = ComputerSystem.BASE_URI + "/" \
            + server_profile["uuid"] + "/EthernetInterfaces/" + conn_id

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = conn_id
        self.redfish["Name"] = connection["name"]
        self.redfish["Status"] = status_mapping \
            .STATUS_MAP[connection["status"]]

        self.redfish["MACAddress"] = connection["mac"]
        if connection["macType"] == "Physical":
            self.redfish["PermanentMACAddress"] = connection["mac"]

        self.redfish["SpeedMbps"] = int(connection["allocatedMbps"])

        if network_attr["category"] == "network-sets":
            self.redfish["VLANs"] = {"@odata.id": odata_uri + "/VLANs"}
        else:
            self.redfish["VLAN"] = dict()
            self.redfish["VLAN"]["VLANEnable"] = True
            self.redfish["VLAN"]["VLANId"] = \
                int(network_attr["attributes"]["vlan_id"])

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#EthernetInterface.EthernetInterface"
        self.redfish["@odata.id"] = odata_uri

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import logging

from flask import Flask
from flask import g
from flask import request
from flask import Response
from flask_api import status
from unittest import mock

from hpOneView import HPOneViewException

from oneview_redfish_toolkit.api.redfish_error import RedfishError
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import handler_multiple_oneview
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.tests.base_test import BaseTest


class BaseFlaskTest(BaseTest):
    """Base class for tests that need a Flask instance"""

    @classmethod
    def setUpClass(cls):
        super(BaseFlaskTest, cls).setUpClass()

        # creates a test client
        cls.app = Flask(cls.__name__)

        cls.oneview_client = mock.MagicMock()

        cls.patcher_get_client_by_token = mock.patch(
            'oneview_redfish_toolkit.client_session.'
            '_get_oneview_client_by_token')
        cls.mock_get_client_by_token = cls.patcher_get_client_by_token.start()

        cls.patcher_get_client_by_ip = mock.patch(
            'oneview_redfish_toolkit.client_session.'
            '_get_oneview_client_by_ip')
        cls.mock_get_client_by_ip = cls.patcher_get_client_by_ip.start()

        multiple_oneview.init_map_resources()

        # same configuration applied to Flask in app.py
        cls.app.url_map.strict_slashes = False

        @cls.app.errorhandler(status.HTTP_500_INTERNAL_SERVER_ERROR)
        def internal_server_error(error):
            """General InternalServerError handler for the app"""

            logging.error(error)

            return ResponseBuilder.error_500(error)

        @cls.app.errorhandler(status.HTTP_403_FORBIDDEN)
        def forbidden(error):
            return ResponseBuilder.error_403(error)

        @cls.app.errorhandler(status.HTTP_404_NOT_FOUND)
        def not_found(error):
            """Creates a Not Found Error response"""
            redfish_error = RedfishError(
                "GeneralError", error.description)
            error_str = redfish_error.serialize()
            return Response(
                response=error_str,
                status=status.HTTP_404_NOT_FOUND,
                mimetype='application/json')

        @cls.app.errorhandler(status.HTTP_400_BAD_REQUEST)
        def bad_request(error):
            """Creates a Bad Request Error response"""
            redfish_error = RedfishError(
                "PropertyValueNotInList", error.description)

            redfish_error.add_extended_info(
                message_id="PropertyValueNotInList",
                message_args=["VALUE", "PROPERTY"],
                related_properties=["PROPERTY"])

            error_str = redfish_error.serialize()
            return Response(
                response=error_str,
                status=status.HTTP_400_BAD_REQUEST,
                mimetype='application/json')

        @cls.app.errorhandler(HPOneViewException)
        def hp_oneview_client_exception(exception):
            logging.exception(exception)
            response = ResponseBuilder.error_by_hp_oneview_exception(exception)

            # checking if session has expired on Oneview
            if config.auth_mode_is_session() and \
                    response.status_code == status.HTTP_401_UNAUTHORIZED:
                token = request.headers.get('x-auth-token')
                client_session.clear_session_by_token(token)

            return response

        @cls.app.before_request
        def check_authentication():
            # Cached OneView's connections for the same request
            g.ov_connections = dict()

            g.elapsed_time_ov = 0

            g.oneview_client = \
                handler_multiple_oneview.MultipleOneViewResource()

        cls.client = cls.app.test_client()

        # propagate the exceptions to the test client
        cls.app.testing = False

    @classmethod
    def setUp(cls):
        cls.oneview_client = mock.MagicMock()
        cls.mock_get_client_by_ip.return_value = cls.oneview_client
        cls.mock_get_client_by_token.return_value = cls.oneview_client
        category_resource.init_map_category_resources()

    @classmethod
    def tearDownClass(cls):
        cls.patcher_get_client_by_ip.stop()
        cls.patcher_get_client_by_token.stop()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
from unittest import mock

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints.subscription_collection\
    import subscription_collection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestSubscriptionCollection(BaseFlaskTest):
    """Tests for SubscriptionCollection blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestSubscriptionCollection, self).setUpClass()

        self.app.register_blueprint(subscription_collection)

    @mock.patch('oneview_redfish_toolkit.util.get_all_subscriptions')
    def test_get_subscription_collection(self, subscriptions_mockup):
        """Tests SubscriptionCollection blueprint result against know value"""

        subscriptions_mockup.return_value = dict()

        response = \
            self.client.get("/redfish/v1/EventService/EventSubscriptions/")

        result = json.loads(response.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/'
            'redfish/SubscriptionCollection.json'
        ) as f:
            subscription_collection_mockup = json.loads(f.read())

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(subscription_collection_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import processor
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestProcessor(BaseFlaskTest):
    """Tests for Processor blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestProcessor, self).setUpClass()

        self.app.register_blueprint(processor.processor)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/Processor.json'
        ) as f:
            self.expected_processor = json.load(f)

    def test_get_processor_server_hardware_not_found(self):
        self.oneview_client.server_hardware.get.side_effect = \
            HPOneViewException({"errorCode": "RESOURCE_NOT_FOUND"})

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/30303437-3034-4D32-3230-313133364752/Systems/1/Processors/1")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_processor_id_not_found(self):
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/30303437-3034-4D32-3230-313133364752/Systems/1/Processors/999")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_processor(self):
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/30303437-3034-4D32-3230-313133364752/Systems/1/Processors/1")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.expected_processor, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for load_schema and load_registry function from util.py
"""

import collections
import configparser

from oneview_redfish_toolkit.api.errors \
    import OneViewRedfishResourceNotFoundError
from oneview_redfish_toolkit.api import schemas
from oneview_redfish_toolkit import config
import unittest


class TestUtil(unittest.TestCase):
    """Test class for config

        Tests:
            load_conf_file()
                - invalid ini file
                - valid ini file
                - have all expected sessions
                - have all expected options
                - checks if globals vars are not none

            load_schema()
                - invalid schema dir

            load_registry()
                - invalid schema dir
                - valid schema dir invalid schemas dict
                - valid schema dir and dict

    """

    # load_conf_file() tests
    def setUp(self):
        self.schema_dir = './oneview_redfish_toolkit/schemas'
        self.registry_dir = './oneview_redfish_toolkit/registry'
        self.config_file = './oneview_redfish_toolkit/conf/redfish.conf'

    def test_load_conf_invalid_config_file(self):
        # Tests if passing a file that does not exists returns false.
        try:
            config.load_conf_file('non-exist.conf')
        except Exception as e:
            self.assertIsInstance(
                e,
                OneViewRedfishResourceNotFoundError
            )

    def test_load_conf_valid_config_file(self):
        # Tests if passing a valid file returns a object
        self.assertIsInstance(config.load_conf_file(self.config_file),
                              configparser.ConfigParser)

    def test_load_conf_has_all_expect_sessions(self):
        # Tests if ini file has all expected sections
        cfg = config.load_conf_file(self.config_file)

        self.assertTrue(cfg.has_section('redfish'),
                        msg='Section {} not found in ini file {}'.format(
                        'redfish', self.config_file))
        self.assertTrue(cfg.has_section('oneview_config'),
                        msg='Section {} not found in ini file {}'.
                        format('oneview_config', self.config_file))
        self.assertTrue(cfg.has_section('credentials'),
                        msg='Section {} not found in ini file {}'.
                        format('credentials', self.config_file))
        self.assertTrue(cfg.has_section('event_service'),
                        msg='Section {} not found in ini file {}'.
                        format('event_service', self.config_file))

    def test_load_conf_has_all_options(self):
        # Tests if ini file has all expected options

        cfg = config.load_conf_file(self.config_file)

        self.assertTrue(cfg.has_option('redfish', 'indent_json'),
                        msg='Option {} not found in section {} in ini file {}'
                        .format('indent_json', 'redfish',
                        self.config_file))
        self.assertTrue(cfg.has_option('oneview_config', 'ip'),
                        msg='Option {} not found in section {} in ini file {}'
                        .format('ip', 'oneview_config', self.config_file))
        self.assertTrue(cfg.has_option('credentials', 'userName'),
                        msg='Option {} not found in section {} in ini file {}'
                        .format('userName', 'credentials',
                        self.config_file))
        self.assertTrue(cfg.has_option('credentials', 'password'),
                        msg='Option {} not found in section {} in ini file {}'
                        .format('password', 'credentials',
                        self.config_file))

    # load_schemas() tests
    def test_load_schemas_invalid_schema_dir(self):
        # Tests if passing a non existing schema dir returns False

        try:
            config.load_schemas('non-exist-schema-dir')
        except Exception as e:
            self.assertIsInstance(
                e,
                OneViewRedfishResourceNotFoundError,
                msg="Unexpected exception: {}".format(e.msg)
            )

    # load_registry() tests
    def test_load_registries_invalid_registry_dir(self):
        # Tests load_registry() passing a non existing registry dir

        schemas = dict()

        try:
            config.load_registry('non-exist-registry-dir', schemas)
        except Exception as e:
            self.assertIsInstance(
                e,
                OneViewRedfishResourceNotFoundError,
                msg="Unexpected exception: {}".format(e.msg)
            )

    def test_load_registries_valid_registry_dir_invalid_dict(self):
        # Tests load_registry() passing a valid registry dir and an invalid
        # registry dict

        registries = dict()
        registries['failed'] = 'fail.json'

        try:
            config.load_registry(self.registry_dir, registries)
        except Exception as e:
            self.assertIsInstance(
                e,
                OneViewRedfishResourceNotFoundError,
                msg="Unexpected exception: {}".format(e.msg)
            )

    def test_load_registries_valid_registry_dir_valid_dict(self):
        # Tests loading registry files from redfish.conf

        registries = schemas.REGISTRY

        try:
            registry_dict = config.load_registry(self.registry_dir, registries)
            self.assertIsInstance(registry_dict, collections.OrderedDict)
        except Exception as e:
            self.fail('Failed to load registries files: {}'.format(e.msg))



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from collections import OrderedDict
import json

# 3rd party libs
from flask_api import status
from unittest import mock

# Module libs
from oneview_redfish_toolkit.blueprints import manager_collection
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestManagerCollection(BaseFlaskTest):
    """Tests for ManagerCollection blueprint

        Tests:
            - oneview unexpected exception
            - know manager collection
    """

    @classmethod
    def setUpClass(self):
        super(TestManagerCollection, self).setUpClass()

        self.app.register_blueprint(
            manager_collection.manager_collection)

    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_manager_collection_unexpected_error(
            self, get_map_appliances):
        """Tests ManagerCollection with an error"""

        get_map_appliances.side_effect = \
            Exception("An exception has occurred")

        with open(
                'oneview_redfish_toolkit/mockups/errors/'
                'Error500.json'
        ) as f:
            error_500 = json.load(f)

        response = self.client.get("/redfish/v1/Managers/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(error_500, result)

    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_manager_collection(self, get_map_appliances):
        """Tests a valid ManagerCollection"""

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'ManagerCollection.json'
        ) as f:
            manager_collection_mockup = json.load(f)

        appliance_info_list = OrderedDict()
        appliance_info_list["10.0.0.1"] = \
            "b08eb206-a904-46cf-9172-dcdff2fa9639"
        appliance_info_list["10.0.0.2"] = \
            "c9ba5ca4-c1f8-48c7-9798-1e8b8897ef50"

        # Create mock response
        get_map_appliances.return_value = appliance_info_list

        # Get ManagerCollection
        response = self.client.get("/redfish/v1/Managers/")

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(manager_collection_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for manager_collection.py
"""

from collections import OrderedDict
import json

from oneview_redfish_toolkit.api.manager_collection \
    import ManagerCollection
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestManagerCollection(BaseTest):
    """Tests for ManagerCollection class"""

    def setUp(self):
        """Tests preparation"""

        # Loading ManagerCollection result mockup
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'ManagerCollection.json'
        ) as f:
            self.manager_collection_mockup = json.load(f)

        self.appliance_info_list = OrderedDict()
        self.appliance_info_list["10.0.0.1"] = \
            "b08eb206-a904-46cf-9172-dcdff2fa9639"
        self.appliance_info_list["10.0.0.2"] = \
            "c9ba5ca4-c1f8-48c7-9798-1e8b8897ef50"

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated

        try:
            manager_collection = ManagerCollection(
                self.appliance_info_list
            )
        except Exception as e:
            self.fail("Failed to instantiate ManagerCollection class."
                      " Error: {}".format(e))
        self.assertIsInstance(manager_collection, ManagerCollection)

    def test_serialize(self):
        # Tests the serialize function result against known result

        try:
            manager_collection = ManagerCollection(
                self.appliance_info_list
            )
        except Exception as e:
            self.fail("Failed to instantiate ManagerCollection class."
                      " Error: {}".format(e))

        try:
            result = json.loads(manager_collection.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: ".format(e))

        self.assertEqualMockup(self.manager_collection_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import copy
import json

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import vlan_network_interface
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestVLanNetworkInterface(BaseFlaskTest):
    """Tests for VLanNetworkInterface blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestVLanNetworkInterface, self).setUpClass()

        self.app.register_blueprint(
            vlan_network_interface.vlan_network_interface)

        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/ServerProfileTemplateNetworkSet.json'
        ) as f:
            self.server_profile_template = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/ServerProfileNetworkSet.json'
        ) as f:
            self.server_profile = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/EthernetNetworkSet.json'
        ) as f:
            self.ethernet_network_mockup = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/NetworkSet.json'
        ) as f:
            self.network_set_mockup = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/VLanNetworkInterface.json'
        ) as f:
            self.expected_vlan_network_interface_spt = json.load(f)

        self.expected_vlan_network_interface_sp = \
            copy.deepcopy(self.expected_vlan_network_interface_spt)
        self.expected_vlan_network_interface_sp["@odata.id"] = \
            "/redfish/v1/Systems" \
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0" \
            "/EthernetInterfaces/1/VLANs/19638712-679d-4232-9743-c7cb6c7bf718"

        self.resource_not_found = HPOneViewException({
            "errorCode": "RESOURCE_NOT_FOUND",
            "message": "Any resource not found message"
        })

    def test_get_spt_vlan_network_interface(self):
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template

        self.oneview_client.ethernet_networks.get.return_value = \
            self.ethernet_network_mockup

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.expected_vlan_network_interface_spt,
                               result)
        self.oneview_client.server_profile_templates.get.assert_called_with(
            self.server_profile_template["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_called_with(
            self.expected_vlan_network_interface_spt["Id"])

    def test_get_spt_vlan_network_interface_not_found(self):
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template

        self.oneview_client.ethernet_networks.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profile_templates.get.assert_called_with(
            self.server_profile_template["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_called_with(
            self.expected_vlan_network_interface_spt["Id"])

    def test_get_spt_vlan_network_interface_spt_not_found(self):
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profile_templates.get.assert_called_with(
            self.server_profile_template["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_not_called()

    def test_get_spt_vlan_network_interface_connection_not_found(self):
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template

        connection_id = "999"

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/" + connection_id +
            "/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profile_templates.get.assert_called_with(
            self.server_profile_template["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_not_called()

    def test_get_spt_vlan_network_interface_collection(self):
        with open(
            'oneview_redfish_toolkit/mockups/'
            'redfish/VLanNetworkInterfaceCollectionSPT.json'
        ) as f:
            expected_vlan_network_interface_collection = json.load(f)

        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template

        self.oneview_client.network_sets.get.return_value = \
            self.network_set_mockup

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0/"
            "EthernetInterfaces/1/VLANs")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_vlan_network_interface_collection,
                               result)
        self.oneview_client.server_profile_templates.get.assert_called_with(
            self.server_profile_template["uri"].split("/")[-1])
        self.oneview_client.network_sets.get.assert_called_with(
            self.network_set_mockup["uri"])

    def test_get_spt_vlan_collection_not_found(self):
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template

        self.oneview_client.network_sets.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profile_templates.get.assert_called_with(
            self.server_profile_template["uri"].split("/")[-1])
        self.oneview_client.network_sets.get.assert_called_with(
            self.network_set_mockup["uri"])

    def test_get_spt_vlan_collection_with_spt_not_found(self):
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profile_templates.get.assert_called_with(
            self.server_profile_template["uri"].split("/")[-1])
        self.oneview_client.network_sets.get.assert_not_called()

    def test_get_sp_vlan_network_interface(self):
        self.oneview_client.server_profiles.get.return_value = \
            self.server_profile

        self.oneview_client.ethernet_networks.get.return_value = \
            self.ethernet_network_mockup

        response = self.client.get(
            "/redfish/v1/Systems"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.expected_vlan_network_interface_sp, result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_called_with(
            self.expected_vlan_network_interface_sp["Id"])

    def test_get_sp_vlan_network_interface_not_found(self):
        self.oneview_client.server_profiles.get.return_value = \
            self.server_profile

        self.oneview_client.ethernet_networks.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/Systems"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_called_with(
            self.expected_vlan_network_interface_sp["Id"])

    def test_get_sp_vlan_network_interface_sp_not_found(self):
        self.oneview_client.server_profiles.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/Systems"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_not_called()

    def test_get_sp_vlan_network_interface_connection_not_found(self):
        self.oneview_client.server_profiles.get.return_value = \
            self.server_profile

        connection_id = "999"

        response = self.client.get(
            "/redfish/v1/Systems"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/" + connection_id +
            "/VLANs/19638712-679d-4232-9743-c7cb6c7bf718")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uri"].split("/")[-1])
        self.oneview_client.ethernet_networks.get.assert_not_called()

    def test_get_sp_vlan_network_interface_collection(self):
        with open(
            'oneview_redfish_toolkit/mockups/'
            'redfish/VLanNetworkInterfaceCollectionSP.json'
        ) as f:
            expected_vlan_network_interface_collection = json.load(f)

        self.oneview_client.server_profiles.get.return_value = \
            self.server_profile

        self.oneview_client.network_sets.get.return_value = \
            self.network_set_mockup

        response = self.client.get(
            "/redfish/v1/Systems"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0/"
            "EthernetInterfaces/1/VLANs")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_vlan_network_interface_collection,
                               result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uri"].split("/")[-1])
        self.oneview_client.network_sets.get.assert_called_with(
            self.network_set_mockup["uri"])

    def test_get_sp_vlan_collection_not_found(self):
        self.oneview_client.server_profiles.get.return_value = \
            self.server_profile

        self.oneview_client.network_sets.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/Systems"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uri"].split("/")[-1])
        self.oneview_client.network_sets.get.assert_called_with(
            self.network_set_mockup["uri"])

    def test_get_sp_vlan_collection_with_sp_not_found(self):
        self.oneview_client.server_profiles.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/Systems"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
            "/EthernetInterfaces/1/VLANs")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uri"].split("/")[-1])
        self.oneview_client.network_sets.get.assert_not_called()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from copy import deepcopy

from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection
import oneview_redfish_toolkit.api.status_mapping as status_mapping
from oneview_redfish_toolkit.api.util.power_option import \
    RESET_ALLOWABLE_VALUES_LIST
from oneview_redfish_toolkit.services.computer_system_service import \
    ComputerSystemService


class ComputerSystem(RedfishJsonValidator):
    """Creates a Computer System Redfish dict

        Populates self.redfish with ComputerSystem data retrieved from oneview
    """

    SCHEMA_NAME = 'ComputerSystem'
    BASE_URI = '/redfish/v1/Systems'

    def __init__(self, server_hardware, server_hardware_types,
                 server_profile, drives, server_profile_template_uuid,
                 manager_uuid):
        """ComputerSystem constructor

            Populates self.redfish with the contents of ServerHardware and
            ServerHardwareTypes dicts.

            Args:
                server_hardware: ServerHardware dict from OneView
                server_hardware_types: ServerHardwareTypes dict from OneView
                server_profile: ServerProfile dict from OneView.
                drives: Drives list from OneView
                server_profile_template_uuid: ServerProfileTemplate uuid
                manager_uuid: Oneview's current manager uuid
        """
        super().__init__(self.SCHEMA_NAME)

        base_resource = server_profile
        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = base_resource["uuid"]
        self.redfish["Description"] = base_resource["description"]
        self.redfish["Name"] = base_resource["name"]
        self.redfish["SystemType"] = "Composed"
        self.redfish["Manufacturer"] = "HPE"
        self.redfish["Model"] = server_hardware["model"]
        self.redfish["SerialNumber"] = server_hardware["serialNumber"]
        self.redfish["Status"] = collections.OrderedDict()
        health = self._get_highest_status_for_sp_and_sh(
            status_mapping.HEALTH_STATE.get(base_resource["status"]),
            status_mapping.HEALTH_STATE.get(server_hardware["status"])
        )
        state, _ = status_mapping.\
            get_redfish_server_profile_state(base_resource)
        self.redfish["Status"]["State"] = state
        self.redfish["Status"]["Health"] = health
        self.redfish["PowerState"] = server_hardware["powerState"]
        self.redfish["Boot"] = collections.OrderedDict()
        self.redfish["Boot"]["BootSourceOverrideTarget@Redfish."
                             "AllowableValues"] = \
            self.map_boot(server_hardware_types['bootCapabilities'])
        self.redfish["BiosVersion"] = server_hardware["romVersion"]
        self.redfish["ProcessorSummary"] = collections.OrderedDict()
        self.redfish["ProcessorSummary"]['Count'] = \
            server_hardware["processorCount"]
        self.redfish["ProcessorSummary"]["Model"] = \
            server_hardware["processorType"]
        self.redfish["MemorySummary"] = collections.OrderedDict()
        self.redfish["MemorySummary"]["TotalSystemMemoryGiB"] = \
            server_hardware["memoryMb"] / 1024
        self.redfish["Storage"] = collections.OrderedDict()
        self.redfish["Storage"]["@odata.id"] = \
            self.BASE_URI + "/" + base_resource['uuid'] + "/Storage"
        self.redfish["EthernetInterfaces"] = collections.OrderedDict()
        self.redfish["EthernetInterfaces"]["@odata.id"] = \
            self.BASE_URI + "/" + \
            base_resource['uuid'] + \
            "/EthernetInterfaces"
        self.redfish["NetworkInterfaces"] = collections.OrderedDict()
        self.redfish["NetworkInterfaces"]["@odata.id"] = \
            self.BASE_URI + "/" + \
            base_resource['uuid'] + \
            "/NetworkInterfaces"
        self.redfish["Links"] = collections.OrderedDict()
        self.redfish["Links"]["Chassis"] = list()
        self.redfish["Links"]["Chassis"].append(collections.OrderedDict())
        self.redfish["Links"]["Chassis"][0]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware['uuid']
        self.redfish["Links"]["ManagedBy"] = list()
        if manager_uuid:
            self.redfish["Links"]["ManagedBy"].append(
                collections.OrderedDict())
            self.redfish["Links"]["ManagedBy"][0]["@odata.id"] = \
                "/redfish/v1/Managers/" + manager_uuid
        self.redfish["Links"]["ResourceBlocks"] = list()
        self._fill_resource_block_members(drives,
                                          server_hardware,
                                          server_profile_template_uuid)
        self.redfish["Actions"] = collections.OrderedDict()
        self.redfish["Actions"]["#ComputerSystem.Reset"] = \
            collections.OrderedDict()
        self.redfish["Actions"]["#ComputerSystem.Reset"]["target"] = \
            self.BASE_URI + "/" + \
            base_resource["uuid"] + \
            "/Actions/ComputerSystem.Reset"
        self.redfish["Actions"]["#ComputerSystem.Reset"][
            "ResetType@Redfish.AllowableValues"] = \
            RESET_ALLOWABLE_VALUES_LIST
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ComputerSystem.ComputerSystem"
        self.redfish["@odata.id"] = self.BASE_URI + "/" \
            + base_resource["uuid"]

        self._validate()

    @staticmethod
    def _get_highest_status_for_sp_and_sh(sp_status, sh_status):
        all_status = dict()
        all_status[sp_status] = \
            status_mapping.CRITICALITY_STATUS[sp_status]
        all_status[sh_status] = \
            status_mapping.CRITICALITY_STATUS[sh_status]

        highest_status = max(all_status, key=(lambda key: all_status[key]))

        return highest_status

    def map_boot(self, boot_list):
        """Maps Oneview's boot options to Redfish's boot option

            Maps the known OneView boot options to Redfish boot option.
            If a unknown boot option shows up it will be mapped to None

            Args:
                boot_list: List with OneView boot options

            Returns:
                list with Redfish boot options
        """

        redfish_oneview_boot_map = dict()
        redfish_oneview_boot_map['PXE'] = 'Pxe'
        redfish_oneview_boot_map['CD'] = 'Cd'
        redfish_oneview_boot_map['HardDisk'] = 'Hdd'
        redfish_oneview_boot_map['FibreChannelHba'] = 'RemoteDrive'
        redfish_oneview_boot_map['Floppy'] = 'Floppy'
        redfish_oneview_boot_map['USB'] = 'Usb'
        redfish_boot_list = list()

        try:
            for boot_option in boot_list:
                redfish_boot_list.append(
                    redfish_oneview_boot_map[boot_option])
        except Exception:
            redfish_boot_list.append('None')

        return redfish_boot_list

    @staticmethod
    def build_server_profile(profile_name,
                             profile_description,
                             server_profile_template,
                             system_block,
                             network_blocks,
                             storage_blocks):
        server_profile = deepcopy(server_profile_template)

        server_profile.pop("uri", None)
        server_profile.pop("serverProfileDescription", None)
        server_profile.pop("created", None)
        server_profile.pop("modified", None)
        server_profile.pop("status", None)
        server_profile.pop("state", None)
        server_profile.pop("scopesUri", None)
        server_profile.pop("eTag", None)
        server_profile.pop("description", None)
        if isinstance(server_profile.get("connectionSettings"), dict):
            server_profile["connectionSettings"].pop('manageConnections', None)

        server_profile["name"] = profile_name
        if profile_description:
            server_profile["description"] = profile_description
        server_profile["type"] = "ServerProfileV8"
        server_profile["category"] = "server-profiles"
        server_profile["serverHardwareUri"] = \
            "/rest/server-hardware/" + system_block["uuid"]
        server_profile["localStorage"]["sasLogicalJBODs"] = \
            ComputerSystem._build_sas_logical_jbods(server_profile_template,
                                                    storage_blocks)

        return server_profile

    @staticmethod
    def _build_sas_logical_jbods(server_profile_template, storage_blocks):
        sas_logical_jbods = []

        controller = ComputerSystemService.get_storage_controller(
            server_profile_template)

        if storage_blocks and not controller:
            raise OneViewRedfishError(
                "The Server Profile Template should have a valid "
                "storage controller to use the Storage Resource Blocks passed")

        for index, storage_block in enumerate(storage_blocks):
            storage_id = index + 1
            attributes = storage_block["attributes"]

            storage = {
                "id": storage_id,
                "name": "Storage " + str(storage_id),
                "deviceSlot": controller["deviceSlot"],
                "numPhysicalDrives": 1,
                "driveMinSizeGB": attributes["capacityInGB"],
                "driveMaxSizeGB": attributes["capacityInGB"],
                "driveTechnology": attributes["interfaceType"].capitalize()
                                   + attributes["mediaType"].capitalize()
            }

            sas_logical_jbods.append(storage)

        return sas_logical_jbods

    def _fill_resource_block_members(self,
                                     drives,
                                     server_hardware,
                                     server_profile_template_uuid):
        resource_block_uuids = \
            self._get_resource_block_uuids(drives,
                                           server_hardware,
                                           server_profile_template_uuid)

        base_uri = ResourceBlockCollection.BASE_URI + "/{}"
        blocks = self.redfish["Links"]["ResourceBlocks"]
        for resource_block_uuid in resource_block_uuids:
            blocks.append({"@odata.id": base_uri.format(resource_block_uuid)})

    def _get_resource_block_uuids(self,
                                  drives,
                                  server_hardware,
                                  server_profile_template_uuid):
        resource_block_uuids = list()
        resource_block_uuids.append(server_hardware["uuid"])

        # fill with network resource block
        if server_profile_template_uuid:
            resource_block_uuids.append(server_profile_template_uuid)

        for drive in drives:
            storage_resource_uuid = drive["uri"].split("/")[-1]
            resource_block_uuids.append(storage_resource_uuid)

        return resource_block_uuids



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from functools import reduce
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask import request
from flask import Response
from flask_api import status

# own libs
from hpOneView.exceptions import HPOneViewException
from hpOneView.exceptions import HPOneViewTaskError
from hpOneView.resources.task_monitor import TASK_ERROR_STATES
from jsonschema import ValidationError

from oneview_redfish_toolkit.api.capabilities_object import CapabilitiesObject
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.util.power_option import OneViewPowerOption
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit.services.computer_system_service import \
    ComputerSystemService
from oneview_redfish_toolkit.services.manager_service import \
    get_manager_uuid

computer_system = Blueprint("computer_system", __name__)


@computer_system.route("/redfish/v1/Systems/<uuid>", methods=["GET"])
def get_computer_system(uuid):
    """Get the Redfish Computer System for a given UUID.

        Return ComputerSystem redfish JSON for a given
        server profile or server profile template UUID.
        Logs exception of OneViewRedfishError and abort(404).

        Returns:
            JSON: Redfish json with ComputerSystem
            When Server hardware or Server profilte templates
            is not found calls abort(404)
    """

    try:
        resource = _get_oneview_resource(uuid)
        category = resource["category"]

        if category == 'server-profile-templates':
            computer_system_resource = CapabilitiesObject(resource)
        elif category == 'server-profiles':
            server_hardware = g.oneview_client.server_hardware\
                .get(resource["serverHardwareUri"])
            server_hardware_type = g.oneview_client.server_hardware_types\
                .get(resource['serverHardwareTypeUri'])

            computer_system_service = ComputerSystemService(g.oneview_client)
            drives = _get_drives_from_sp(resource)
            spt_uuid = computer_system_service.\
                get_server_profile_template_from_sp(resource["uri"])

            manager_uuid = get_manager_uuid(resource['serverHardwareTypeUri'])

            # Build Computer System object and validates it
            computer_system_resource = ComputerSystem(server_hardware,
                                                      server_hardware_type,
                                                      resource,
                                                      drives,
                                                      spt_uuid,
                                                      manager_uuid)
        else:
            raise OneViewRedfishError(
                'Computer System UUID {} not found'.format(uuid))

        return ResponseBuilder.success(
            computer_system_resource,
            {"ETag": "W/" + resource["eTag"]})
    except OneViewRedfishError as e:
        # In case of error log exception and abort
        logging.exception('Unexpected error: {}'.format(e))
        abort(status.HTTP_404_NOT_FOUND, e.msg)


@computer_system.route("/redfish/v1/Systems/<uuid>/"
                       "Actions/ComputerSystem.Reset", methods=["POST"])
def change_power_state(uuid):
    """Change the Oneview power state for a specific Server hardware.

        Return ResetType Computer System redfish JSON for a
        given server profile UUID.
        Logs exception of any error and return abort.

        Returns:
            JSON: Redfish JSON with ComputerSystem ResetType.

        Exceptions:
            HPOneViewException: When some OneView resource was not found.
            return abort(404)

            OneViewRedfishError: When occur a power state mapping error.
            return abort(400)

            Exception: Unexpected error.
            return abort(500)
    """

    try:
        try:
            reset_type = request.get_json()["ResetType"]
        except KeyError:
            invalid_key = list(request.get_json())[0]  # gets invalid key name
            raise OneViewRedfishError(
                {"errorCode": "INVALID_INFORMATION",
                 "message": "Invalid JSON key: {}".format(invalid_key)})

        # Gets ServerHardware for given UUID
        profile = g.oneview_client.server_profiles.get(uuid)
        sh = g.oneview_client.server_hardware.get(profile["serverHardwareUri"])

        oneview_power_configuration = \
            OneViewPowerOption.get_oneview_power_configuration(
                sh, reset_type)

        # Changes the ServerHardware power state
        g.oneview_client.server_hardware.update_power_state(
            oneview_power_configuration, sh["uuid"])

        return Response(
            response='{"ResetType": "%s"}' % reset_type,
            status=status.HTTP_200_OK,
            mimetype='application/json')

    except OneViewRedfishError as e:
        # In case of error log exception and abort
        logging.exception('Mapping error: {}'.format(e))

        if e.msg["errorCode"] == "NOT_IMPLEMENTED":
            abort(status.HTTP_501_NOT_IMPLEMENTED, e.msg['message'])
        else:
            abort(status.HTTP_400_BAD_REQUEST, e.msg['message'])


@computer_system.route(
    "/redfish/v1/Systems/<uuid>", methods=["DELETE"])
def remove_computer_system(uuid):
    """Removes a specific System

        Args:
            uuid: The System ID.
    """

    profile = g.oneview_client.server_profiles.get(uuid)
    sh_uuid = profile['serverHardwareUri']
    if sh_uuid:
        service = ComputerSystemService(g.oneview_client)
        try:
            service.power_off_server_hardware(sh_uuid)
        except OneViewRedfishError as e:
            abort(status.HTTP_403_FORBIDDEN, e.msg)

    # Deletes server profile for given UUID
    response = None
    try:
        response = g.oneview_client.server_profiles.delete(uuid)
    except HPOneViewTaskError as e:
        abort(status.HTTP_403_FORBIDDEN, e.msg)

    if response is True:
        return Response(status=status.HTTP_204_NO_CONTENT,
                        mimetype="application/json")

    # Check if returned a task
    if type(response) is dict:
        # Check if task is completed
        if response['taskState'] == 'Completed':
            return Response(status=status.HTTP_204_NO_CONTENT,
                            mimetype="application/json")

        # Log task error messages if it has
        if response['taskState'] in TASK_ERROR_STATES and \
            'taskErrors' in response and len(response['taskErrors']) > 0:
            for err in response['taskErrors']:
                if 'message' in err:
                    logging.exception(err['message'])

    abort(status.HTTP_500_INTERNAL_SERVER_ERROR)


@computer_system.route(ComputerSystem.BASE_URI + "/", methods=["POST"])
def create_composed_system():
    if not request.is_json:
        abort(status.HTTP_400_BAD_REQUEST,
              "The request content should be a valida JSON")

    body = request.get_json()
    result_location_uri = None

    try:
        RedfishJsonValidator.validate(body, 'ComputerSystem')
        service = ComputerSystemService(g.oneview_client)

        blocks = body["Links"]["ResourceBlocks"]
        block_ids = [block["@odata.id"].split("/")[-1] for block in blocks]

        # Should contain only one computer system entry
        system_blocks = _get_system_resource_blocks(block_ids)
        if not system_blocks:
            raise ValidationError(
                "Should have a Computer System Resource Block")

        system_block = system_blocks[0]
        service.validate_computer_system_resource_block_to_composition(
            system_block)

        # Check network block id with the Id attribute in the request
        network_blocks = _get_network_resource_blocks(block_ids)
        spt_id = body["Id"]

        if not (network_blocks and spt_id in network_blocks[0]["uri"]):
            raise ValidationError(
                "Should have a valid Network Resource Block")

        # It can contain zero or more Storage Block
        storage_blocks = _get_storage_resource_blocks(block_ids)

        spt = g.oneview_client.server_profile_templates.get(spt_id)

        server_profile = ComputerSystem.build_server_profile(
            body["Name"],
            body.get("Description"),
            spt,
            system_block,
            network_blocks,
            storage_blocks)

        service.power_off_server_hardware(system_block["uuid"],
                                          on_compose=True)

        task, resource_uri = service.create_composed_system(server_profile)

        if resource_uri:
            result_uuid = resource_uri.split("/")[-1]
            result_location_uri = ComputerSystem.BASE_URI + "/" + result_uuid
            server_profile_label = dict(
                resourceUri=resource_uri, labels=[spt_id.replace("-", " ")])
            g.oneview_client.labels.create(server_profile_label)
        elif task.get("taskErrors"):
            err_msg = reduce(
                lambda result, msg: result + msg["message"] + "\n",
                task["taskErrors"],
                "")
            raise OneViewRedfishError(err_msg)

    except ValidationError as e:
        abort(status.HTTP_400_BAD_REQUEST, e.message)
    except KeyError as e:
        abort(status.HTTP_400_BAD_REQUEST,
              "Trying access an invalid key {}".format(e.args))
    except (HPOneViewTaskError, OneViewRedfishError) as e:
        abort(status.HTTP_403_FORBIDDEN, e.msg)

    if not result_location_uri:
        logging.error("It was not possible get the server profile URI when "
                      "creating a composed system")
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR)

    return Response(status=status.HTTP_201_CREATED,
                    headers={"Location": result_location_uri},
                    mimetype="application/json")


def _get_oneview_resource(uuid):
    """Gets a Server hardware or Server profile templates"""
    cached_category = category_resource.get_category_by_resource_id(uuid)

    if cached_category:
        resource = getattr(g.oneview_client, cached_category.resource)
        function = getattr(resource, cached_category.function)

        return function(uuid)

    categories = [
        {"func": g.oneview_client.server_profiles.get, "param": uuid},
        {"func": g.oneview_client.server_profile_templates.get, "param": uuid}
    ]

    for category in categories:
        try:
            resource = category["func"](category["param"])

            return resource
        except HPOneViewException as e:
            if e.oneview_response["errorCode"] in \
                    ['RESOURCE_NOT_FOUND', 'ProfileNotFoundException']:
                pass
            else:
                raise  # Raise any unexpected errors

    raise OneViewRedfishError("Could not find computer system with id " + uuid)


def _get_system_resource_blocks(ids):
    return _get_resource_block_data(
        g.oneview_client.server_hardware.get, ids)


def _get_network_resource_blocks(ids):
    return _get_resource_block_data(
        g.oneview_client.server_profile_templates.get, ids)


def _get_storage_resource_blocks(ids):
    drive_ids = ["/rest/drives/" + i for i in ids]

    return _get_resource_block_data(
        g.oneview_client.index_resources.get, drive_ids)


def _get_resource_block_data(func, uuids):
    resources = list()

    for uuid in uuids:
        try:
            resource = func(uuid)

            resources.append(resource)
        except HPOneViewException as e:
            # With our current implementation, we do not getting
            # server_profile, but if we need get it in some moment,
            # we must check the errorCode 'ProfileNotFoundException'
            if e.oneview_response["errorCode"] == 'RESOURCE_NOT_FOUND':
                pass
            else:
                raise  # Raise any unexpected errors

    return resources


def _get_drives_from_sp(server_profile):
    """Gets Drives from Server Profile"""
    jbods_drives = list()
    for sas_logical_jbod in server_profile["localStorage"]["sasLogicalJBODs"]:
        if sas_logical_jbod["sasLogicalJBODUri"]:
            drives_by_sas_logical_uuid = g.oneview_client.sas_logical_jbods. \
                get_drives(sas_logical_jbod["sasLogicalJBODUri"])
            jbods_drives.extend(drives_by_sas_logical_uuid)

    return jbods_drives



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from flask_api import status

from oneview_redfish_toolkit.blueprints import computer_system_collection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestComputerSystemCollection(BaseFlaskTest):
    """Tests for ComputerSystemCollection blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestComputerSystemCollection, self).setUpClass()

        self.app.register_blueprint(
            computer_system_collection.computer_system_collection)

    def test_get_computer_system_collection_empty(self):
        """Tests ComputerSystemCollection with an empty list"""

        self.oneview_client.server_hardware.get_all.return_value = []
        self.oneview_client.server_profile_templates.get_all.return_value = []

        response = self.client.get("/redfish/v1/Systems/")

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'ServerProfilesAppliedCollectionEmpty.json'
        ) as f:
            expected_result = json.load(f)

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_result, result)

    def test_get_computer_system_collection_fail(self):
        """Tests ComputerSystemCollection with an error"""

        self.oneview_client.server_hardware.get_all.side_effect = \
            Exception("An exception has occurred")

        with open(
                'oneview_redfish_toolkit/mockups/errors/'
                'Error500.json'
        ) as f:
            error_500 = json.load(f)

        response = self.client.get("/redfish/v1/Systems/")

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(error_500, result)

    def test_get_computer_system_collection(self):
        """Tests ComputerSystemCollection with a known Server Hardware list"""

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerHardwareListWithServerProfileApplied.json'
        ) as f:
            server_hardware_list = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerProfileTemplates.json'
        ) as f:
            server_profile_template_list = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/LogicalEnclosures.json'
        ) as f:
            logical_encl_list = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'ComputerSystemCollection.json'
        ) as f:
            computer_system_collection_mockup = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'DriveEnclosureList.json'
        ) as f:
            drive_enclosure = json.load(f)
        self.oneview_client.server_hardware.get_all.return_value = \
            server_hardware_list

        self.oneview_client.server_profile_templates.get_all.return_value = \
            server_profile_template_list

        self.oneview_client.logical_enclosures.get_all.return_value = \
            logical_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            drive_enclosure

        response = self.client.get("/redfish/v1/Systems/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(computer_system_collection_mockup, result)

        self.oneview_client.server_hardware.get_all.assert_called_with(
            filter="NOT 'serverProfileUri' = NULL")
        self.oneview_client.\
            server_profile_templates.get_all.assert_called_with()

        self.oneview_client.drive_enclosures.get_all.assert_called_with()
        self.oneview_client.logical_enclosures.get_all.assert_called_with()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import storage_collection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestStorageCollection(BaseFlaskTest):
    """Tests for StorageCollection blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestStorageCollection, self).setUpClass()

        self.app.register_blueprint(storage_collection.storage_collection)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerProfile.json'
        ) as f:
            self.server_profile = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/StorageCollection.json'
        ) as f:
            self.storage_collection_mockup = json.load(f)

    def test_get_storage_collection(self):
        """Tests StorageCollection"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage"
        )

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.storage_collection_mockup, result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])

    def test_get_storage_collection_when_profile_not_found(self):
        """Tests StorageCollection"""

        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'server-hardware not found',
        })
        self.oneview_client.server_profiles.get.side_effect = e

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])

    def test_get_storage_collection_when_profile_raises_any_exception(self):
        """Tests StorageCollection"""

        e = HPOneViewException({
            'errorCode': 'ANOTHER_ERROR',
            'message': 'server-hardware-types error',
        })
        self.oneview_client.server_profiles.get.side_effect = e

        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import copy
import json
from unittest import mock

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints import composition_service
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestCompositionService(BaseFlaskTest):
    """Tests for CompositionService blueprint

        Tests:
            - composition service serialization exception
            - known composition service resource
    """

    @classmethod
    def setUpClass(self):
        super(TestCompositionService, self).setUpClass()

        self.app.register_blueprint(composition_service.composition_service)

    @mock.patch.object(composition_service.CompositionService, 'serialize')
    def test_get_composition_service_unexpected_error(self, mock):
        """Tests CompositionService with an error"""

        mock.side_effect = Exception()

        with open(
                'oneview_redfish_toolkit/mockups/errors/'
                'Error500.json'
        ) as f:
            error_500 = json.load(f)

        error_500_excep = copy.deepcopy(error_500)
        error_500_excep["error"]["message"] = \
            "The server encountered an internal error and " \
            "was unable to complete your request.  " \
            "Either the server is overloaded or there is " \
            "an error in the application."

        response = self.client.get("/redfish/v1/CompositionService/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(error_500_excep, result)

    def test_get_composition_service(self):
        """Tests CompositionService"""

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'CompositionService.json'
        ) as f:
            composition_service_mockup = json.load(f)

        # Get CompositionService
        response = self.client.get("/redfish/v1/CompositionService/")

        # Gets json from response
        expected_result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(composition_service_mockup, expected_result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from flask_api import status

from oneview_redfish_toolkit.blueprints import storage_composition_details
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestStorageCompositionDetails(BaseFlaskTest):
    """Tests for StorageCompositionDetails blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestStorageCompositionDetails, self).setUpClass()

        self.app.register_blueprint(
            storage_composition_details.storage_composition_details)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Drive.json'
        ) as f:
            self.drive = json.load(f)

    def test_get_storage_details(self):
        self.oneview_client.index_resources.get.return_value = self.drive

        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'StorageCompositionDetails.json'
        ) as f:
            expected_storage_details = json.load(f)

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e/Storage/1")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_storage_details, result)

    def test_get_storage_details_when_it_is_not_found(self):
        self.oneview_client.index_resources.get.return_value = self.drive

        wrong_id = "2"  # any value other than "1"

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e/Storage/" + wrong_id)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

        msg_error = "Storage 2 not found for ResourceBlock " \
                    "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e"
        self.assertIn(msg_error, str(result))

    def test_get_storage_drive_details(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'StorageDriveCompositionDetails.json'
        ) as f:
            expected_drive_details = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/DriveEnclosure.json'
        ) as f:
            drive_enclosure = json.load(f)

        self.oneview_client.index_resources.get.return_value = self.drive
        self.oneview_client.drive_enclosures.get.return_value = drive_enclosure

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e/Storage/1/Drives/1")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_drive_details, result)

    def test_get_storage_drive_details_when_drive_is_not_found(self):
        self.oneview_client.index_resources.get.return_value = self.drive

        wrong_id = "2"  # any value other than "1"

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e/Storage/1/Drives/"
            + wrong_id)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

        msg_error = "Drive 2 not found for Storage 1 of ResourceBlock " \
                    "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e"
        self.assertIn(msg_error, str(result))

    def test_get_storage_drive_details_when_storage_is_not_found(self):
        self.oneview_client.index_resources.get.return_value = self.drive

        wrong_id = "2"  # any value other than "1"

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e/Storage/{}/Drives/1"
            .format(wrong_id))

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

        msg_error = "Storage 2 not found for ResourceBlock " \
                    "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e"
        self.assertIn(msg_error, str(result))



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
from unittest import mock

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import service_root
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestServiceRoot(BaseFlaskTest):
    """Tests from ServiceRoot blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestServiceRoot, self).setUpClass()

        self.app.register_blueprint(
            service_root.service_root, url_prefix='/redfish/v1/')

    @mock.patch.object(service_root, 'connection')
    def test_service_root_oneview_exception(self, connection):
        """Tests ServiceROOT with an exception"""

        e = HPOneViewException({
            'errorCode': 'ANOTHER_ERROR',
            'message': 'appliance error',
        })

        connection.request_oneview.side_effect = e

        response = self.client.get("/redfish/v1/")

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )

        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(config, 'get_authentication_mode')
    @mock.patch.object(config, 'get_oneview_multiple_ips')
    @mock.patch.object(config, 'get_credentials')
    @mock.patch.object(connection, 'request_oneview')
    @mock.patch.object(client_session, 'get_oneview_client')
    def test_get_service_root_when_auth_mode_is_session(
            self,
            get_oneview_client,
            request_oneview,
            get_credentials,
            get_oneview_multiple_ips,
            get_authentication_mode):
        """Tests get ServiceRoot when authentication mode is session"""

        get_authentication_mode.return_value = 'session'
        request_oneview.return_value = \
            {'uuid': '00000000-0000-0000-0000-000000000000'}
        get_oneview_multiple_ips.return_value = ['10.0.0.1']
        get_credentials.return_value = {'userName': '',
                                        'password': ''}

        result = self.client.get("/redfish/v1/")

        result = json.loads(result.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/redfish/ServiceRoot.json'
        ) as f:
            service_root_mockup = json.load(f)

        self.assertEqualMockup(service_root_mockup, result)
        get_credentials.assert_not_called()
        get_oneview_client.assert_not_called()

    @mock.patch.object(config, 'get_authentication_mode')
    @mock.patch.object(config, 'get_oneview_multiple_ips')
    @mock.patch.object(config, 'get_credentials')
    @mock.patch.object(connection, 'request_oneview')
    @mock.patch.object(client_session, 'get_oneview_client')
    def test_get_service_root_when_auth_mode_is_conf(self,
                                                     get_oneview_client,
                                                     request_oneview,
                                                     get_credentials,
                                                     get_oneview_multiple_ips,
                                                     get_authentication_mode):
        """Tests get ServiceRoot when authentication mode is conf"""

        get_authentication_mode.return_value = 'conf'
        request_oneview.return_value = \
            {'uuid': '00000000-0000-0000-0000-000000000000'}
        get_oneview_multiple_ips.return_value = ['10.0.0.1']
        get_credentials.return_value = {'userName': '',
                                        'password': ''}

        result = self.client.get("/redfish/v1/")

        result = json.loads(result.data.decode("utf-8"))

        with open(
                'oneview_redfish_toolkit/mockups/redfish/ServiceRoot.json'
        ) as f:
            service_root_mockup = json.load(f)
            service_root_mockup['Links']['Sessions'] = {}

        self.assertEqualMockup(service_root_mockup, result)
        get_credentials.assert_not_called()
        get_oneview_client.assert_not_called()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from unittest import mock

from oneview_redfish_toolkit.api.event import Event
from oneview_redfish_toolkit.api.subscription import Subscription
from oneview_redfish_toolkit.event_dispatcher import EventDispatcher
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestEventDispatcher(BaseTest):
    """Tests for event_dispatcher.py"""

    @mock.patch('http.client.HTTPConnection.request')
    @mock.patch('logging.exception')
    def test_dispatch_event(self, exception_mock, request_mock):
        """Tests dispatch event"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Alert.json'
        ) as f:
            event_mockup = json.loads(f.read())

            event = Event(event_mockup)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/EventDestination.json'
        ) as f:
            subscription_mockup = json.loads(f.read())

            subscription = Subscription(
                subscription_mockup['Id'],
                subscription_mockup['Destination'],
                subscription_mockup['EventTypes'],
                subscription_mockup['Context'])

        dispatcher = EventDispatcher(event, subscription, 1, 1)

        dispatcher.run()

        self.assertTrue(request_mock.called)
        self.assertFalse(exception_mock.called)

    @mock.patch.object(Event, 'serialize')
    @mock.patch('http.client.HTTPConnection.request')
    @mock.patch('logging.exception')
    def test_dispatch_event_serialization_fail(
        self, exception_mock, request_mock, serialize_mock):
        """Tests dispatch event with serialization fail"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Alert.json'
        ) as f:
            event_mockup = json.loads(f.read())

            event = Event(event_mockup)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/EventDestination.json'
        ) as f:
            subscription_mockup = json.loads(f.read())

            subscription = Subscription(
                subscription_mockup['Id'],
                subscription_mockup['Destination'],
                subscription_mockup['EventTypes'],
                subscription_mockup['Context'])

        dispatcher = EventDispatcher(event, subscription, 1, 1)

        serialize_mock.side_effect = Exception()

        dispatcher.run()

        self.assertFalse(request_mock.called)
        self.assertTrue(exception_mock.called)

    @mock.patch('time.sleep')
    @mock.patch('http.client.HTTPConnection.request')
    @mock.patch('logging.exception')
    def test_dispatch_event_request_fail(
        self, exception_mock, request_mock, sleep_mock):
        """Tests dispatch event with request fail"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Alert.json'
        ) as f:
            event_mockup = json.loads(f.read())

            event = Event(event_mockup)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/EventDestination.json'
        ) as f:
            subscription_mockup = json.loads(f.read())

            subscription = Subscription(
                subscription_mockup['Id'],
                subscription_mockup['Destination'],
                subscription_mockup['EventTypes'],
                subscription_mockup['Context'])

        dispatcher = EventDispatcher(event, subscription, 1, 1)

        request_mock.side_effect = Exception()

        dispatcher.run()

        sleep_mock.assert_called_with(1)
        self.assertTrue(exception_mock.called)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
from unittest import mock

# 3rd party libs
from unittest.mock import call

from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import resource_block
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit import single_oneview_context
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestSingleOneViewContext(BaseFlaskTest):
    """Tests for ResourceBlock blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestSingleOneViewContext, self).setUpClass()

        self.app.register_blueprint(resource_block.resource_block)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Drive.json'
        ) as f:
            self.drive = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/DriveIndexTrees.json'
        ) as f:
            self.drive_index_tree = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/ServerProfileTemplates.json'
        ) as f:
            self.server_profile_templates = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/LogicalEnclosures.json'
        ) as f:
            self.log_encl_list = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview'
                '/DriveEnclosureList.json'
        ) as f:
            self.drive_enclosure_list = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/StorageResourceBlock.json'
        ) as f:
            self.expected_resource_block = json.load(f)

        self.resource_not_found = HPOneViewException({
            "errorCode": "RESOURCE_NOT_FOUND",
            "message": "Any resource not found message"
        })

    @mock.patch.object(config, 'get_oneview_multiple_ips')
    def test_get_storage_resource_block_single_ov(self,
                                                  get_oneview_multiple_ips):
        multiple_oneview.init_map_resources()
        category_resource.init_map_category_resources()

        list_ips = ['10.0.0.1', '10.0.0.2', '10.0.0.3']
        get_oneview_multiple_ips.return_value = list_ips

        self.oneview_client.server_hardware.get.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.resource_not_found,
            ]
        self.oneview_client.server_profile_templates.get.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.resource_not_found,
            ]
        self.oneview_client.index_resources.get.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.drive,
            ]
        self.oneview_client.connection.get.return_value = self.drive_index_tree
        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.\
            logical_enclosures.get_all.return_value = self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        uri = "/redfish/v1/CompositionService/ResourceBlocks"\
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e"
        uuid = uri.split('/')[-1]
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.expected_resource_block, result)

        self.oneview_client.server_hardware.get.assert_has_calls([
            call(uuid),
            call(uuid),
            call(uuid)
            ])
        self.oneview_client.server_profile_templates.get.assert_has_calls([
            call(uuid),
            call(uuid),
            call(uuid)
            ])
        self.oneview_client.index_resources.get.assert_has_calls([
            call(self.drive["uri"]),
            call(self.drive["uri"]),
            call(self.drive["uri"]),
            ])
        conn_uri = "/rest/index/trees/rest/drives/"\
            "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e?parentDepth=3"
        # Check for single calls on OneView context
        self.oneview_client.connection.get.assert_called_once_with(
            conn_uri)
        self.oneview_client.\
            server_profile_templates.get_all.assert_called_once_with()
        self.oneview_client.logical_enclosures.get_all.\
            assert_called_once_with()
        self.oneview_client.drive_enclosures.get_all.\
            assert_called_once_with()

        ov_ip_cached_drive = multiple_oneview.get_ov_ip_by_resource(
            self.drive['uri'])
        self.assertEqual(ov_ip_cached_drive, list_ips[2])

        ov_ip_cached_conn_drive = multiple_oneview.get_ov_ip_by_resource(
            conn_uri)
        self.assertTrue(ov_ip_cached_conn_drive, list_ips[2])

    @mock.patch.object(config, 'get_oneview_multiple_ips')
    @mock.patch.object(single_oneview_context, 'is_single_oneview_context')
    def test_get_storage_resource_block_without_single_ov(self,
                                                          is_single_ov_context,
                                                          get_ov_multiple_ips):
        multiple_oneview.init_map_resources()
        category_resource.init_map_category_resources()

        list_ips = ['10.0.0.1', '10.0.0.2', '10.0.0.3']
        get_ov_multiple_ips.return_value = list_ips

        is_single_ov_context.return_value = False

        self.oneview_client.server_hardware.get.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.resource_not_found,
            ]
        self.oneview_client.server_profile_templates.get.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.resource_not_found,
            ]
        self.oneview_client.index_resources.get.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.drive,
            ]
        self.oneview_client.connection.get.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.drive_index_tree
            ]
        self.oneview_client.server_profile_templates.get_all.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.server_profile_templates
            ]
        self.oneview_client.logical_enclosures.get_all.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.log_encl_list
            ]
        self.oneview_client.drive_enclosures.get_all.side_effect = [
            self.resource_not_found,
            self.resource_not_found,
            self.drive_enclosure_list
            ]

        uri = "/redfish/v1/CompositionService/ResourceBlocks"\
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e"
        uuid = uri.split('/')[-1]
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.expected_resource_block, result)

        self.oneview_client.server_hardware.get.assert_has_calls([
            call(uuid),
            call(uuid),
            call(uuid)
            ])
        self.oneview_client.server_profile_templates.get.assert_has_calls([
            call(uuid),
            call(uuid),
            call(uuid)
            ])
        self.oneview_client.index_resources.get.assert_has_calls([
            call(self.drive["uri"]),
            call(self.drive["uri"]),
            call(self.drive["uri"]),
            ])
        conn_uri = "/rest/index/trees/rest/drives/"\
            "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e?parentDepth=3"
        # Check for multiple calls on multiple OneView context
        self.oneview_client.connection.get.assert_has_calls(
            [call(conn_uri), call(conn_uri), call(conn_uri)])
        self.oneview_client.server_profile_templates.get_all.has_calls(
            [call(), call(), call()])
        self.oneview_client.logical_enclosures.get_all.has_calls(
            [call(), call(), call()])
        self.oneview_client.drive_enclosures.get_all.has_calls(
            [call(), call(), call()])

        ov_ip_cached_drive = multiple_oneview.get_ov_ip_by_resource(
            self.drive['uri'])
        self.assertEqual(ov_ip_cached_drive, list_ips[2])

        ov_ip_cached_conn_drive = multiple_oneview.get_ov_ip_by_resource(
            conn_uri)
        self.assertTrue(ov_ip_cached_conn_drive, list_ips[2])



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection
import oneview_redfish_toolkit.api.status_mapping as status_mapping


class ResourceBlockComputerSystem(RedfishJsonValidator):
    """Creates a Resource Block Computer System Redfish dict

        Populates self.redfish with some predefined Computer System
        contents and with data retrieved from OneView
    """

    SCHEMA_NAME = 'ComputerSystem'

    def __init__(self, server_hardware, manager_uuid):
        """ResourceBlock ComputerSystem constructor

            Populates self.redfish with the contents of server hardware dict.

            Args:
                server_hardware: server hardware dict from OneView
                manager_uuid: Oneview's current manager
        """
        super().__init__(self.SCHEMA_NAME)

        self.server_hardware = server_hardware

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = server_hardware["uuid"]
        self.redfish["Name"] = server_hardware["name"]
        self.redfish["SystemType"] = "Physical"
        self.redfish["Manufacturer"] = "HPE"
        self.redfish["Model"] = server_hardware["model"]
        self.redfish["SerialNumber"] = server_hardware["serialNumber"]
        self.redfish["Status"] = collections.OrderedDict()
        state, health = status_mapping.\
            get_redfish_server_hardware_status_struct(server_hardware)
        self.redfish["Status"]["State"] = state
        self.redfish["Status"]["Health"] = health
        self.redfish["PowerState"] = server_hardware["powerState"]
        self.redfish["BiosVersion"] = server_hardware["romVersion"]

        self.redfish["ProcessorSummary"] = collections.OrderedDict()
        self.redfish["ProcessorSummary"]['Count'] = \
            server_hardware["processorCount"]
        self.redfish["ProcessorSummary"]["Model"] = \
            server_hardware["processorType"]
        self.redfish["Processors"] = dict()
        self.redfish["Processors"]["@odata.id"] = \
            ResourceBlockCollection.BASE_URI + "/" \
            + server_hardware["uuid"] + "/Systems/1/Processors"

        self.redfish["MemorySummary"] = collections.OrderedDict()
        self.redfish["MemorySummary"]["TotalSystemMemoryGiB"] = \
            server_hardware["memoryMb"] / 1024

        self._fill_links(manager_uuid)

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ComputerSystem.ComputerSystem"
        self.redfish["@odata.id"] = \
            ResourceBlockCollection.BASE_URI + "/" \
            + server_hardware["uuid"] \
            + "/Systems/1"

        self._validate()

    def _fill_links(self, manager_uuid):
        self.redfish["Links"] = collections.OrderedDict()

        self.redfish["Links"]["Chassis"] = list()
        chassi = dict()
        chassi["@odata.id"] = "/redfish/v1/Chassis/{}" \
            .format(self.server_hardware["uuid"])
        self.redfish["Links"]["Chassis"].append(chassi)

        self.redfish["Links"]["ManagedBy"] = list()
        if manager_uuid:
            manager = dict()
            manager["@odata.id"] = "/redfish/v1/Managers/" \
                                   + manager_uuid
            self.redfish["Links"]["ManagedBy"].append(manager)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

from oneview_redfish_toolkit.api.storage_resource_block import \
    StorageResourceBlock
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestStorageResourceBlock(BaseTest):
    """Tests for StorageResourceBlock class"""

    @classmethod
    def setUpClass(self):
        """Tests preparation"""

        super(TestStorageResourceBlock, self).setUpClass()

        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/StorageResourceBlock.json'
        ) as f:
            self.resource_block_mockup = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Drive.json'
        ) as f:
            self.drive = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/DriveIndexTrees.json'
        ) as f:
            self.drive_index_tree = json.load(f)

    def test_serialize(self):
        zone_ids = [
            "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0-0000000000A66101",
            "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0-0000000000A66102",
            "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0-0000000000A66103",
            "75871d70-789e-4cf9-8bc8-6f4d73193578",
        ]

        resource_block = StorageResourceBlock(
            self.drive, self.drive_index_tree, zone_ids)
        result = json.loads(resource_block.serialize())

        self.assertEqualMockup(self.resource_block_mockup, result)

    def test_when_server_profile_template_has_not_storage_controller(self):
        """This test is a invalid state (it should never occur)

            The Drive ever must have at least one Zone
        """
        expected_result = copy.copy(self.resource_block_mockup)
        expected_result["Links"]["Zones"] = []

        zone_ids = ["75871d70-789e-4cf9-8bc8-6f4d73193578"]

        resource_block = StorageResourceBlock(
            self.drive, self.drive_index_tree, zone_ids)
        result = json.loads(resource_block.serialize())

        self.assertEqualMockup(self.resource_block_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask_api import status

from oneview_redfish_toolkit.blueprints.redfish_base import redfish_base
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestRedfishBase(BaseFlaskTest):

    @classmethod
    def setUpClass(self):
        super(TestRedfishBase, self).setUpClass()

        self.app.register_blueprint(redfish_base, url_prefix='/redfish/')

    def test_get_redfish_base_status(self):
        # sends HTTP GET request to the application
        # on the specified path
        result = self.client.get("/redfish/")

        # assert the status code of the response
        self.assertEqual(result.status_code, status.HTTP_200_OK)

    def test_get_redfish_base_response(self):
        result = self.client.get("/redfish/")

        json_str = result.data.decode("utf-8")

        self.assertEqual(json_str, '{"v1": "/redfish/v1/"}')



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.capabilities_object import CapabilitiesObject
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestCapabilitiesObject(BaseTest):
    """Tests for CapabilitiesObject class"""

    with open('oneview_redfish_toolkit/mockups/oneview/'
              'ServerProfileTemplate.json') as f:
        server_profile_template = json.load(f)

    with open('oneview_redfish_toolkit/mockups/redfish/'
              'CapabilitiesObject.json') as f:
        capabilities_mockup = json.load(f)

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated
        capabilities_obj = CapabilitiesObject(self.server_profile_template)

        self.assertIsInstance(capabilities_obj, CapabilitiesObject)

    def test_serialize(self):
        # Tests the serialize function result against known result
        capabilities_obj = CapabilitiesObject(self.server_profile_template)
        result = json.loads(capabilities_obj.serialize())

        self.assertEqualMockup(self.capabilities_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
from unittest import mock

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints import session_service
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


@mock.patch.object(config, 'auth_mode_is_session')
class TestSessionService(BaseFlaskTest):
    """Tests for SessionService blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestSessionService, self).setUpClass()

        self.app.register_blueprint(session_service.session_service)

    def test_get_session_service_when_is_enabled(self, auth_mode_is_session):
        """Tests get SessionService when is enabled"""
        auth_mode_is_session.return_value = True

        response = self.client.get("/redfish/v1/SessionService")

        result = json.loads(response.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/redfish/SessionService.json'
        ) as f:
            session_service_mockup = json.loads(f.read())

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(session_service_mockup, result)

    def test_get_session_service_when_is_disabled(self, auth_mode_is_session):
        """Tests get SessionService when is disabled"""
        auth_mode_is_session.return_value = False

        response = self.client.get("/redfish/v1/SessionService")

        result = json.loads(response.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'SessionServiceDisabled.json'
        ) as f:
            session_service_mockup = json.loads(f.read())

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(session_service_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection


class ResourceBlockEthernetInterface(RedfishJsonValidator):
    """Creates a Ethernet Interface Redfish dict

        Populates self.redfish with ethernet interface data retrieved
        from OneView
    """

    SCHEMA_NAME = 'EthernetInterface'

    def __init__(self, server_profile_template, connection, network):
        """ResourceBlockEthernetInterface constructor

            Populates self.redfish with some hardcoded Resource Block
            values and with Server Profile Template and Ethernet Network
            data retrieved from OneView.

            Args:
                server_profile_template: Server Profile Template dict
                from OneView
                connection: Server Profile Template Connection dict
                from OneView
                network: Ethernet network dict from OneView
        """

        super().__init__(self.SCHEMA_NAME)

        server_profile_template_id = \
            server_profile_template["uri"].split("/")[-1]
        odata_id = ResourceBlockCollection.BASE_URI + "/" \
            + server_profile_template_id \
            + "/EthernetInterfaces/" \
            + str(connection["id"])

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = server_profile_template_id
        self.redfish["Name"] = network["name"]
        self.redfish["SpeedMbps"] = int(connection["requestedMbps"])

        if network["category"] == "ethernet-networks":
            self.redfish["VLAN"] = \
                self._create_vlan(network["attributes"]["vlan_id"])
        elif network["category"] == "network-sets":
            self.redfish["VLANs"] = dict()
            self.redfish["VLANs"]["@odata.id"] = \
                odata_id \
                + "/VLANs"

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#EthernetInterface.EthernetInterface"
        self.redfish["@odata.id"] = odata_id

        self._validate()

    def _create_vlan(self, vlan_id):
        vlan = dict()
        vlan["VLANEnable"] = True
        vlan["VLANId"] = int(vlan_id)
        return vlan



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for store_schema and load_registry function from util.py
"""

from oneview_redfish_toolkit import connection
import unittest
from unittest import mock


class TestUtil(unittest.TestCase):
    """Test class for util

        Tests:
            get_oneview_client()
                - connection recover
                - connection renew
                - connection failure
    """

    # load_conf() tests
    def setUp(self):
        self.schema_dir = './oneview_redfish_toolkit/schemas'
        self.registry_dir = './oneview_redfish_toolkit/registry'
        self.config_file = './oneview_redfish_toolkit/conf/redfish.conf'

    @mock.patch.object(connection, 'OneViewClient')
    def test_get_ov_client_recover(self, oneview_client_mockup):
        # Tests a successful recover of a OV client

        oneview_client = oneview_client_mockup()
        oneview_client.connection.get.return_value = list()

        try:
            ov_client = connection.OneViewClient({})
        except Exception as e:
            self.fail('Failed to connect to OneView: '.format(e))
        self.assertIsNotNone(ov_client)

    @mock.patch.object(connection, 'OneViewClient')
    def test_get_ov_client_renew(self, oneview_client_mockup):
        """Tests getting a OV client from expired session"""

        oneview_client = oneview_client_mockup()
        oneview_client.connection.get.return_value = \
            Exception('session expired')
        oneview_client.connection.login.return_value = oneview_client

        try:
            ov_client = connection.OneViewClient({})
        except Exception as e:
            self.fail('Failed to connect to OneView: '.format(e))
        self.assertIsNotNone(ov_client)

    @mock.patch.object(connection, 'OneViewClient')
    def test_get_ov_client_oneview_offline(self, oneview_client_mockup):
        """Tests getting a OV client from an offline oneview"""

        oneview_client = oneview_client_mockup()
        oneview_client.connection.get.return_value = \
            Exception('OneView not responding')
        oneview_client.connection.login.return_value = \
            Exception('OneView not responding')

        try:
            ov_client = connection.OneViewClient({})
        except Exception as e:
            self.fail('Failed to connect to OneView: '.format(e))
        self.assertIsNotNone(ov_client)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints import chassis_collection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestChassisCollection(BaseFlaskTest):
    """Tests for ChassisCollection blueprint

        Tests:
            - server hardware empty
            - enclosures empty
            - racks empty
            - oneview unexpected exception
            - know chassis collection
    """

    @classmethod
    def setUpClass(self):
        super(TestChassisCollection, self).setUpClass()

        self.app.register_blueprint(chassis_collection.chassis_collection)

    def test_get_chassis_collection_unexpected_error(self):
        """Tests ChassisCollection with an error"""

        self.oneview_client.enclosures.get_all.return_value = [object]
        self.oneview_client.racks.get_all.return_value = [object]
        self.oneview_client.server_hardware.get_all.side_effect = \
            Exception("An exception has occurred")

        with open(
                'oneview_redfish_toolkit/mockups/errors/'
                'Error500.json'
        ) as f:
            error_500 = json.load(f)

        response = self.client.get("/redfish/v1/Chassis/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(error_500, result)

    def test_get_chassis_collection_empty(self):
        """Tests ChassisCollection with an empty sh, enclosure and rack list"""

        self.oneview_client.enclosures.get_all.return_value = []
        self.oneview_client.racks.get_all.return_value = []
        self.oneview_client.server_hardware.get_all.return_value = []

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'ChassisCollectionEmpty.json'
        ) as f:
            chassis_collection_empty = json.load(f)

        response = self.client.get("/redfish/v1/Chassis/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(chassis_collection_empty, result)

    def test_get_chassis_collection(self):
        """Tests ChassisCollection with a known Results"""

        # Loading server_hardware mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerHardwareList.json'
        ) as f:
            server_hardware_list = json.load(f)

        # Loading enclosures mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'Enclosures.json'
        ) as f:
            enclosures = json.load(f)

        # Loading racks mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'Racks.json'
        ) as f:
            racks = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'ChassisCollection.json'
        ) as f:
            chassis_collection_mockup = json.load(f)

        # Create mock response
        self.oneview_client.server_hardware.get_all.return_value = \
            server_hardware_list
        self.oneview_client.enclosures.get_all.return_value = enclosures
        self.oneview_client.racks.get_all.return_value = racks

        # Get ChassisCollection
        response = self.client.get("/redfish/v1/Chassis/")

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(chassis_collection_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
from unittest.mock import call

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import thermal
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestChassis(BaseFlaskTest):
    """Tests for Thermal blueprint

        Tests:
            - blades
            - enclosures
            - racks
    """

    @classmethod
    def setUpClass(self):
        super(TestChassis, self).setUpClass()

        self.app.register_blueprint(thermal.thermal)

    #############
    # Blade     #
    #############

    def test_get_blade_thermal(self):
        """"Tests BladeThermal with a known SH"""

        # Loading ServerHardwareUtilization mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerHardwareUtilization.json'
        ) as f:
            server_hardware_utilization = json.load(f)

        # Loading BladeChassisThermal mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'BladeChassisThermal.json'
        ) as f:
            blade_chassis_thermal_mockup = json.load(f)

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get_utilization.return_value = \
            server_hardware_utilization

        # Get BladeThermal
        response = self.client.get(
            "/redfish/v1/Chassis/36343537-3338-4448-3538-4E5030333434/Thermal"
        )

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(blade_chassis_thermal_mockup, result)

    def test_get_blade_thermal_cached(self):
        """"Tests BladeThermal with a known SH"""

        # Loading ServerHardwareUtilization mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerHardwareUtilization.json'
        ) as f:
            server_hardware_utilization = json.load(f)

        # Loading BladeChassisThermal mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'BladeChassisThermal.json'
        ) as f:
            blade_chassis_thermal_mockup = json.load(f)

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get_utilization.return_value = \
            server_hardware_utilization

        uri = "/redfish/v1/Chassis/36343537-3338-4448-3538-4E5030333434/"\
              "Thermal"
        uuid = "36343537-3338-4448-3538-4E5030333434"

        # Get BladeThermal
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(blade_chassis_thermal_mockup, result)

        # Get cached BladeThermal
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(blade_chassis_thermal_mockup, result)

        # Check for cached calls
        self.oneview_client.index_resources.get_all.assert_called_once_with(
            filter='uuid=' + uuid
        )
        assert self.oneview_client.server_hardware.get.has_calls(
            [call(uuid),
             call(uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))

    def test_get_blade_not_found(self):
        """Tests BladeThermal with SH not found"""

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get_utilization.return_value = \
            {'serverHardwareUri': 'invalidUri'}
        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'server hardware not found',
        })

        self.oneview_client.server_hardware.get_utilization.side_effect = e

        response = self.client.get(
            "/redfish/v1/Chassis/36343537-3338-4448-3538-4E5030333434/Thermal"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_blade_unexpected_error(self):
        """Tests BladeThermal with an unexpected error"""

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get_utilization.side_effect = \
            Exception()

        response = self.client.get(
            "/redfish/v1/Chassis/36343537-3338-4448-3538-4E5030333434/Thermal"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    #############
    # Enclosure #
    #############

    def test_get_encl_thermal(self):
        """"Tests EnclosureThermal with a known Enclosure"""

        # Loading EnclosureUtilization mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'EnclosureUtilization.json'
        ) as f:
            enclosure_utilization = json.load(f)

        # Loading EnclosureChassisThermal mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'EnclosureChassisThermal.json'
        ) as f:
            enclosure_chasssis_thermal_mockup = json.load(f)

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]
        self.oneview_client.enclosures.get_utilization.return_value = \
            enclosure_utilization

        # Get EnclosureThermal
        response = self.client.get(
            "/redfish/v1/Chassis/0000000000A66101/Thermal"
        )

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(enclosure_chasssis_thermal_mockup, result)

    def test_get_encl_thermal_cached(self):
        """"Tests EnclosureThermal with a known Enclosure"""

        # Loading EnclosureUtilization mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'EnclosureUtilization.json'
        ) as f:
            enclosure_utilization = json.load(f)

        # Loading EnclosureChassisThermal mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'EnclosureChassisThermal.json'
        ) as f:
            enclosure_chasssis_thermal_mockup = json.load(f)

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]
        self.oneview_client.enclosures.get_utilization.return_value = \
            enclosure_utilization

        uri = "/redfish/v1/Chassis/0000000000A66101/Thermal"
        uuid = "0000000000A66101"

        # Get EnclosureThermal
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(enclosure_chasssis_thermal_mockup, result)

        # Get cached EnclosureThermal
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(enclosure_chasssis_thermal_mockup, result)

        # Check for cached calls
        self.oneview_client.index_resources.get_all.assert_called_once_with(
            filter='uuid=' + uuid
        )
        assert self.oneview_client.enclosures.get.has_calls(
            [call(uuid),
             call(uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))

    ########
    # Rack #
    ########

    def test_get_rack_thermal(self):
        """"Tests RackThermal with a known Rack"""

        # Loading RackDeviceTopology mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'RackDeviceTopology.json'
        ) as f:
            rack_topology = json.load(f)

        # Loading RackChassisThermal mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/RackChassisThermal.json'
        ) as f:
            rack_chassis_thermal_mockup = json.load(f)

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "racks"}]
        self.oneview_client.\
            racks.get_device_topology.return_value = rack_topology

        # Get RackThermal
        response = self.client.get(
            "/redfish/v1/Chassis/2AB100LMNB/Thermal"
        )

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(rack_chassis_thermal_mockup, result)

    def test_get_rack_thermal_cached(self):
        """"Tests RackThermal with a known Rack"""

        # Loading RackDeviceTopology mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'RackDeviceTopology.json'
        ) as f:
            rack_topology = json.load(f)

        # Loading RackChassisThermal mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/RackChassisThermal.json'
        ) as f:
            rack_chassis_thermal_mockup = json.load(f)

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "racks"}]
        self.oneview_client.\
            racks.get_device_topology.return_value = rack_topology

        uri = "/redfish/v1/Chassis/2AB100LMNB/Thermal"
        uuid = '2AB100LMNB'

        # Get RackThermal
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(rack_chassis_thermal_mockup, result)

        # Get cached RackThermal
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(rack_chassis_thermal_mockup, result)

        # Check for cached calls
        self.oneview_client.index_resources.get_all.assert_called_once_with(
            filter='uuid=' + uuid
        )
        assert self.oneview_client.rackes.get.has_calls(
            [call(uuid),
             call(uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))
